﻿using SharpChat.Misuzu;
using SharpChat.Packet;
using System;
using System.Linq;
using System.Threading.Tasks;

namespace SharpChat.Commands {
    public class BanListCommand : IUserCommand {
        private readonly MisuzuClient Misuzu;

        public BanListCommand(MisuzuClient msz) {
            Misuzu = msz;
        }

        public bool IsMatch(UserCommandContext ctx) {
            return ctx.NameEquals("bans")
                || ctx.NameEquals("banned");
        }

        public void Dispatch(UserCommandContext ctx) {
            if(!ctx.User.Permissions.HasFlag(UserPermissions.KickUser)
                && !ctx.User.Permissions.HasFlag(UserPermissions.BanUser)) {
                ctx.Chat.SendTo(ctx.User, new CommandNotAllowedErrorPacket(ctx.Name));
                return;
            }

            Task.Run(async () => {
                ctx.Chat.SendTo(ctx.User, new BanListResponsePacket(
                    (await Misuzu.GetBanListAsync() ?? Array.Empty<MisuzuBanInfo>()).Select(
                        ban => string.IsNullOrEmpty(ban.UserName) ? (string.IsNullOrEmpty(ban.RemoteAddress) ? string.Empty : ban.RemoteAddress) : ban.UserName
                    ).ToArray()
                ));
            }).Wait();
        }
    }
}
