﻿using SharpChat.Packet;
using System.Linq;

namespace SharpChat.Commands {
    public class ChannelDeleteCommand : IUserCommand {
        public bool IsMatch(UserCommandContext ctx) {
            return ctx.NameEquals("delchan") || (
                ctx.NameEquals("delete")
                && ctx.Args.FirstOrDefault()?.All(char.IsDigit) == false
            );
        }

        public void Dispatch(UserCommandContext ctx) {
            if(!ctx.Args.Any() || string.IsNullOrWhiteSpace(ctx.Args.FirstOrDefault())) {
                ctx.Chat.SendTo(ctx.User, new CommandFormatErrorPacket());
                return;
            }

            string delChanName = string.Join('_', ctx.Args);
            ChannelInfo? delChan = ctx.Chat.Channels.Get(delChanName, SockChatUtility.SanitiseChannelName);

            if(delChan == null) {
                ctx.Chat.SendTo(ctx.User, new ChannelNotFoundErrorPacket(delChanName));
                return;
            }

            if(!ctx.User.Permissions.HasFlag(UserPermissions.DeleteChannel) || !delChan.IsOwner(ctx.User)) {
                ctx.Chat.SendTo(ctx.User, new ChannelDeleteNotAllowedErrorPacket(delChan.Name));
                return;
            }

            ctx.Chat.RemoveChannel(delChan);
            ctx.Chat.SendTo(ctx.User, new ChannelDeleteResponsePacket(delChan.Name));
        }
    }
}
