﻿using SharpChat.Events;
using SharpChat.Packet;
using System;
using System.Linq;

namespace SharpChat.Commands {
    public class MessageWhisperCommand : IUserCommand {
        public bool IsMatch(UserCommandContext ctx) {
            return ctx.NameEquals("whisper")
                || ctx.NameEquals("msg");
        }

        public void Dispatch(UserCommandContext ctx) {
            if(ctx.Args.Length < 2) {
                ctx.Chat.SendTo(ctx.User, new CommandFormatErrorPacket());
                return;
            }

            string whisperUserStr = ctx.Args.FirstOrDefault() ?? string.Empty;
            (string name, UsersContext.NameTarget target) = SockChatUtility.ExplodeUserName(whisperUserStr);
            UserInfo? whisperUser = ctx.Chat.Users.Get(name: name, nameTarget: target);

            if(whisperUser == null) {
                ctx.Chat.SendTo(ctx.User, new UserNotFoundErrorPacket(whisperUserStr));
                return;
            }

            if(whisperUser == ctx.User)
                return;

            ctx.Chat.DispatchEvent(new MessageCreateEvent(
                SharpId.Next(),
                UserInfo.GetDMChannelName(ctx.User, whisperUser),
                ctx.User,
                DateTimeOffset.Now,
                string.Join(' ', ctx.Args.Skip(1)),
                true, false, false
            ));
        }
    }
}
