﻿using SharpChat.Misuzu;
using SharpChat.Packet;
using System.Linq;
using System.Net;
using System.Threading.Tasks;

namespace SharpChat.Commands {
    public class PardonAddressCommand : IUserCommand {
        private readonly MisuzuClient Misuzu;

        public PardonAddressCommand(MisuzuClient msz) {
            Misuzu = msz;
        }

        public bool IsMatch(UserCommandContext ctx) {
            return ctx.NameEquals("pardonip")
                || ctx.NameEquals("unbanip");
        }

        public void Dispatch(UserCommandContext ctx) {
            if(!ctx.User.Permissions.HasFlag(UserPermissions.KickUser)
                && !ctx.User.Permissions.HasFlag(UserPermissions.BanUser)) {
                ctx.Chat.SendTo(ctx.User, new CommandNotAllowedErrorPacket(ctx.Name));
                return;
            }

            string? unbanAddrTarget = ctx.Args.FirstOrDefault();
            if(string.IsNullOrWhiteSpace(unbanAddrTarget) || !IPAddress.TryParse(unbanAddrTarget, out IPAddress? unbanAddr)) {
                ctx.Chat.SendTo(ctx.User, new CommandFormatErrorPacket());
                return;
            }

            unbanAddrTarget = unbanAddr.ToString();

            Task.Run(async () => {
                MisuzuBanInfo? banInfo = await Misuzu.CheckBanAsync(ipAddr: unbanAddrTarget);

                if(banInfo?.IsBanned != true || banInfo.HasExpired) {
                    ctx.Chat.SendTo(ctx.User, new KickBanNoRecordErrorPacket(unbanAddrTarget));
                    return;
                }

                bool wasBanned = await Misuzu.RevokeBanAsync(banInfo, MisuzuClient.BanRevokeKind.RemoteAddress);
                if(wasBanned)
                    ctx.Chat.SendTo(ctx.User, new PardonResponsePacket(unbanAddrTarget));
                else
                    ctx.Chat.SendTo(ctx.User, new KickBanNoRecordErrorPacket(unbanAddrTarget));
            }).Wait();
        }
    }
}
