﻿using SharpChat.Packet;
using System.Linq;

namespace SharpChat.Commands {
    public class UserNickCommand : IUserCommand {
        public bool IsMatch(UserCommandContext ctx) {
            return ctx.NameEquals("nick");
        }

        public void Dispatch(UserCommandContext ctx) {
            bool setOthersNick = ctx.User.Permissions.HasFlag(UserPermissions.SetOthersNickname);

            if(!setOthersNick && !ctx.User.Permissions.HasFlag(UserPermissions.SetOwnNickname)) {
                ctx.Chat.SendTo(ctx.User, new CommandNotAllowedErrorPacket(ctx.Name));
                return;
            }

            UserInfo? targetUser = null;
            int offset = 0;

            if(setOthersNick && long.TryParse(ctx.Args.FirstOrDefault(), out long targetUserId) && targetUserId > 0) {
                targetUser = ctx.Chat.Users.Get(targetUserId);
                ++offset;
            }

            targetUser ??= ctx.User;

            if(ctx.Args.Length < offset) {
                ctx.Chat.SendTo(ctx.User, new CommandFormatErrorPacket());
                return;
            }

            string nickStr = string.Join('_', ctx.Args.Skip(offset))
                .Replace("\n", string.Empty).Replace("\r", string.Empty)
                .Replace("\f", string.Empty).Replace("\t", string.Empty)
                .Replace(' ', '_').Trim();

            if(nickStr == targetUser.UserName)
                nickStr = string.Empty;
            else if(nickStr.Length > 15)
                nickStr = nickStr[..15];
            else if(string.IsNullOrEmpty(nickStr))
                nickStr = string.Empty;

            if(!string.IsNullOrWhiteSpace(nickStr) && ctx.Chat.Users.Get(name: nickStr, nameTarget: UsersContext.NameTarget.UserAndNickName) != null) {
                ctx.Chat.SendTo(ctx.User, new UserNameInUseErrorPacket(nickStr));
                return;
            }

            string? previousName = targetUser == ctx.User ? (targetUser.NickName ?? targetUser.UserName) : null;
            ctx.Chat.UpdateUser(targetUser, nickName: nickStr, silent: previousName == null);
        }
    }
}
